package org.vamsas.client.simpleclient;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.util.Hashtable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.vamsas.client.Events;

/**
 * monitors watcher objects and generates events.
 */
public class EventGeneratorThread extends Thread implements Runnable {
  private static Log log = LogFactory.getLog(EventGeneratorThread.class);
  private SimpleClient client;
  private Hashtable handlers; // manager object
  private VamsasSession session;
  
  protected FileWatcher clientfile=null;
  protected FileWatcher vamsasfile=null;
  protected FileWatcher storeFile=null;
  
  private boolean watch=false;
  
  
  EventGeneratorThread(VamsasSession s, SimpleClient _client, Hashtable eventhandlers) {
    if (eventhandlers==null || s==null || _client==null)
      throw new Error("Null arguments to EventGeneratorThread constructor.");
    handlers = eventhandlers;
    session = s;
    client = _client;
    setName(s.sessionDir.getName());
    initWatchers();
  }
  
  private void initWatchers() {
    if (clientfile==null)
      clientfile = session.getClientWatcher();
    if (vamsasfile ==null)
      vamsasfile = session.getDocWatcher();
    if (storeFile == null)
      storeFile = session.getStoreWatcher();
    clientfile.setState();
    vamsasfile.setState();
    storeFile.setState();
  }
  boolean ownsf = false;
  /**
   * scans all watchers and fires changeEvents if necessary
   * @return number of events generated.
   */
  private int checkforEvents() {
    Lock watchlock;
    //TODO : leave slog.info messages for the events that occur.
    int raised=0;
    // could make this general - but for now keep simple
    if ((watchlock=storeFile.getChangedState())!=null) {
      // TODO: define the storeFile semaphore mechanism : file exists - all clients inform their apps, and then the client that wrote the file should delete the file (it should hold the lock to it).
      if (storeFile.exists) { 
        PropertyChangeSupport h = (PropertyChangeSupport) handlers.get(Events.DOCUMENT_FINALIZEAPPDATA);
        if (h!=null) {
          log.debug("Triggering DOCUMENT_FINALIZEAPPDATA");
          raised++;
          h.firePropertyChange(client.getSessionUrn(), null, client);
          // expect client to 
          vamsasfile.setState();
        }
      }
    }
    if ((watchlock=clientfile.getChangedState())!=null) {
      // see what happened to the clientfile - compare our internal version with the one in the file, or just send the updated list out...?
      raised++;
    }
    if ((watchlock=vamsasfile.getChangedState())!=null) {
      // pass IClientDocument instance to app handler ?
    }
    return raised;
  }
  
  private void initEvents() {
    
  }
  
  /**
   * probably don't need any of these below.
   */
  /* (non-Javadoc)
   * @see java.lang.Thread#destroy()
   */
  public void destroy() {
    super.destroy();
  }
  /* (non-Javadoc)
   * @see java.lang.Thread#interrupt()
   */
  public void interrupt() {
    // TODO Auto-generated method stub
    super.interrupt();
  }
  /* (non-Javadoc)
   * @see java.lang.Thread#isInterrupted()
   */
  public boolean isInterrupted() {
    // TODO Auto-generated method stub
    return super.isInterrupted();
  }
  /* (non-Javadoc)
   * @see java.lang.Thread#run()
   */
  public void run() {
    // TODO Auto-generated method stub
    super.run();
  }
  

}
